@@ -73,10 +73,15 @@ cforest <- function
    
     ### get the call and the calling environment for .urp_tree
     call <- match.call(expand.dots = FALSE)
-    m <- match(c("formula", "data", "subset", "na.action", "weights", "offset", "cluster", 
+    oweights <- NULL
+    if (!missing(weights))
+        oweights <- weights
+    m <- match(c("formula", "data", "subset", "na.action", "offset", "cluster", 
                  "scores", "ytrafo", "control"), names(call), 0L)
     ctreecall <- call[c(1L, m)]
     ctreecall$doFit <- FALSE
+    if (!is.null(oweights))
+        ctreecall$weights <- 1:NROW(oweights)
     ctreecall$control <- control ### put ... into ctree_control()
     ctreecall[[1L]] <- quote(partykit::ctree)
     tree <- eval(ctreecall, parent.frame())
@@ -97,34 +102,64 @@ cforest <- function
     }
     
     probw <- NULL
-    weights <- model.weights(model.frame(d))
+    iweights <- model.weights(model.frame(d))
+    if (!is.null(oweights)) {
+        if (is.matrix(oweights)) {
+            weights <- oweights[iweights,,drop = FALSE]
+        } else {
+            weights <- oweights[iweights]
+        }
+    } else {
+        weights <- NULL
+    }
+    rm(oweights)
+    rm(iweights)
+    N <- nrow(model.frame(d))
+    rw <- NULL
     if (!is.null(weights)) {
-        probw <- weights / sum(weights)
+        if (is.matrix(weights)) {
+            if (ncol(weights) == ntree && nrow(weights) == N) {
+                rw <- unclass(as.data.frame(weights))
+                rw <- lapply(rw, function(w) 
+                    rep(1:length(w), w))
+                weights <- integer(0)
+            } else {
+                stop(sQuote("weights"), "argument incorrect")
+            }
+        } else {
+            probw <- weights / sum(weights)
+        }
     } else {
         weights <- integer(0)
     }
 
-    N <- nrow(model.frame(d))
     idx <- .start_subset(d)
-    if (is.null(strata)) {
-        size <- N
-        if (!perturb$replace) size <- floor(size * perturb$fraction)
-        rw <- replicate(ntree, sample(idx, size = size, replace = perturb$replace, prob = probw),
-                        simplify = FALSE)
-    } else {
-        frac <- if (!perturb$replace) perturb$fraction else 1
-        rw <- replicate(ntree, function() 
-            do.call("c", tapply(idx, strata, function(i) sample(i, size = length(i) * frac, 
-                   replace = perturb$replace, prob = probw[i]))))
+    if (is.null(rw)) {
+        if (is.null(strata)) {
+            size <- N
+            if (!perturb$replace) size <- floor(size * perturb$fraction)
+            rw <- replicate(ntree, 
+                            sample(idx, size = size, 
+                                   replace = perturb$replace, prob = probw),
+                            simplify = FALSE)
+        } else {
+            frac <- if (!perturb$replace) perturb$fraction else 1
+            rw <- replicate(ntree, function() 
+                  do.call("c", tapply(idx, strata, 
+                          function(i) 
+                              sample(i, size = length(i) * frac, 
+                                     replace = perturb$replace, prob = probw[i]))))
+        }
     }
 
     ## apply infrastructure for determining split points
+    ## use RNGkind("L'Ecuyer-CMRG") to make this reproducible
     if (is.null(applyfun)) {
         applyfun <- if(is.null(cores)) {
             lapply  
         } else {
             function(X, FUN, ...)
-                parallel::mclapply(X, FUN, ..., mc.cores = cores)
+                parallel::mclapply(X, FUN, ..., mc.set.seed = TRUE, mc.cores = cores)
         }
     }
 
@@ -133,7 +168,7 @@ cforest <- function
     forest <- applyfun(1:ntree, function(b) {
         if (trace) setTxtProgressBar(pb, b/ntree)
         ret <- updatefun(sort(rw[[b]]), integer(0), control)
-        trafo <<- ret$trafo
+        # trafo <<- ret$trafo
         ret$nodes
     })
     if (trace) close(pb)
@@ -169,8 +204,11 @@ predict.cforest <- function(object, newdata = NULL, type = c("response", "prob",
     vmatch <- 1:ncol(nd)
     NOnewdata <- TRUE
     if (!is.null(newdata)) {
+        factors <- which(sapply(nd, is.factor))
+        xlev <- lapply(factors, function(x) levels(nd[[x]]))
+        names(xlev) <- names(nd)[factors]
         nd <- model.frame(object$predictf, ### all variables W/O response
-                          data = newdata, na.action = na.pass)
+                          data = newdata, na.action = na.pass, xlev = xlev)
         OOB <- FALSE
         vmatch <- match(names(object$data), names(nd))
         NOnewdata <- FALSE
